module.exports = function(RED) {
    "use strict";
    	
	var ffi = require('ffi');

	var libnrbin = ffi.Library('libnrbin', {
	    'bin0_value': [ 'int', [ ] ],
	    'bin1_value': [ 'int', [ ] ],
            'out0_set': [ 'int', ['int'] ],
	    'out1_set': [ 'int', ['int'] ]
	});

	var binPins = ["P1", "P2"];

    function BinaryInputNode(n) {
        // Create a RED node
        RED.nodes.createNode(this,n);

        var node = this;

	    // Store local copies of the node configuration (as defined in the .html)
        this.topic = n.topic;
        this.pin = n.pin;
        
        if (n.activeLow) {    // Set the 'active' state 0 or 1 as appropriate
            this.activeState = 0;
        } else {
            this.activeState = 1;
        }
        
        if (n.outputOn === "rising") {
            this.activeEdges = [false, true];
        } else if (n.outputOn === "falling") {
            this.activeEdges = [true, false];
        } else if (n.outputOn === "both") {
            this.activeEdges = [true, true];
        } else {
            node.error("Invalid edge type: " + n.outputOn);
        }

        this.updateInterval = n.updateInterval * 1000;    // How often to send totalActiveTime messages
	    // Working variables
        this.intervalId = null;         // Remember the timer ID so we can delete it when we are closed
        this.currentState = 0;          // The pin input state "1" or "0"

	    var timerCallback = function () {
	    
            if (binPins.indexOf(node.pin) >= 0) {
            
                if(node.pin === 'P1') {
                    node.currentState = libnrbin.bin0_value();
                }else if(node.pin === 'P2') {
                    node.currentState = libnrbin.bin1_value();
                } 

                    if (node.currentState !== node.activeState) {
                        
                        if(node.activeEdges[node.currentState]){

                            var msg = {};
                            msg.topic = node.topic;
                            msg.payload = Number(node.currentState);
                            node.send(msg);              
                        
                        }
                        
                        node.activeState = node.currentState;

                    }

                }           
          
        };

	node.intervalId = setInterval(timerCallback, node.updateInterval);

        this.on("close", function() {
            // Called when the node is shutdown - eg on redeploy.
            // Allows ports to be closed, connections dropped etc.
            // eg: node.client.disconnect();
        });
    }

    function BinaryOutputNode(n) {
        // Create a RED node
        RED.nodes.createNode(this,n);

        var node = this;

  	    // Store local copies of the node configuration (as defined in the .html)
        this.topic = n.topic;                           // the topic is not currently used
        this.pin = n.pin;                               // The Beaglebone Black pin identifying string
        //this._pin = adjustName(this.pin);             // Adjusted for Octal if necessary
        this.defaultState = Number(n.defaultState);     // What state to set up as
        this.inverting = n.inverting;
        this.toggle = n.toggle;

        // Working variables
        this.currentState = this.defaultState;		

	    var inputCallback = function (msg) {
            var newState;
            if (msg.payload) {
                    newState = true;
                } else {
                    newState = false;
                }
                if (node.inverting) {
                    newState = !newState;
                }
		
	        libnrbin.out0_set(newState ? 1 : 0);
	        node.send({topic: node.topic, payload: newState});
            node.currentState = newState;

        };

	    node.on("input", inputCallback);
	    libnrbin.out0_set(this.defaultState ? 1 : 0);
  
        this.on("close", function() {
            // Called when the node is shutdown - eg on redeploy.
            // Allows ports to be closed, connections dropped etc.
            // eg: node.client.disconnect();
        });
        
    }    

    // Register the node by name. This must be called before overriding any of the
    // Node functions.
    RED.nodes.registerType("binaryInput",BinaryInputNode);
    RED.nodes.registerType("binaryOutput",BinaryOutputNode);

}